/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.configuration.settings.internal.util;

import aQute.bnd.annotation.metatype.Meta;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

/**
 * @author Iván Zaera
 */
public class ConfigurationPidUtil {

	public static String getConfigurationPid(Class<?> configurationBeanClass) {
		for (Annotation annotation : configurationBeanClass.getAnnotations()) {
			Class<? extends Annotation> clazz = annotation.annotationType();

			String name = clazz.getName();

			if (name.equals(Meta.OCD.class.getName())) {
				try {
					Method method = clazz.getMethod("id");

					method.setAccessible(true);

					return (String)method.invoke(annotation);
				}
				catch (ReflectiveOperationException roe) {
					throw new IllegalArgumentException(
						"Unable to obtain configuration PID", roe);
				}
			}
		}

		throw new IllegalArgumentException(
			"Invalid configuration bean class: " +
				configurationBeanClass.getName());
	}

}